{
 07.08.1999
   - Added
     * prperty BarStyle;
   - Changed
     * drawing a gauge in the last panel
 20.08.1999
   - Added
     * prperty CenterText
}
unit VVMSB;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls;

type
  TProcentChange=procedure (Sender: TObject; Percent: Integer) of object;
  TShowStyle=(ssNone,ssPercent,ssText,ssBoth);
  TVVMStatusbarStyle=(vsbColor, vsbEdged);
  TVVMStatusBar = class(TStatusBar)
  private
    FRect : TRect;
    FOldRect: TRect;
    FMax,FMin,FCurrent: LongInt;
    FNumPanel: Integer;
    FBarColor: TColor;
    FOnChangePerc: TProcentChange;
    FCaption: String;
    FCaptStyle: TShowStyle;
    FStyle    : TVVMStatusbarStyle;
    FPercent  : Integer;
    FCentered : Boolean;
    FFull     : Boolean;
    { Private declarations }
  protected
    { Protected declarations }
    Procedure PaintPanel(const Rect: TRect);
    procedure SetMax(Val: Longint);
    procedure SetMin(Val: Longint);
    procedure SetPosition(Val: Longint);
    Procedure SetStyle(Val: TShowStyle);
    procedure SetPanel(val: Integer);
    procedure SetCaption(val: String);
    procedure SetBarColor(val: TColor);
    Function  GetPercent: Integer;
    Procedure SetFStyle(Value: TVVMStatusbarStyle);
    Procedure SetCentered(Value: Boolean);
  public
    procedure DrawPanel(Panel: TStatusPanel; const Rect: TRect);override;
    Constructor Create(AOwner: TComponent); override;
    Procedure StepIt;
    Procedure Invalidate; override;
    { Public declarations }
  published
    Property BarColor: TColor read FBarColor write SetBarColor
                       default clBlue;
    Property Max : LongInt read FMax write setMax default 100;
    Property Min : LongInt read FMin write setMin default 0;
    Property Position : LongInt read FCurrent write SetPosition default 0;
    Property Percent : Integer read GetPercent;
    property PanelNumber: Integer read FNumPanel write SetPanel;
    property OnPercentChanged: TProcentChange read FOnChangePerc
                    write FOnChangePerc;
    Property Caption: String read FCaption write SetCaption;
    Property CaptionStyle: TShowStyle read FCaptStyle write SetStyle default ssPercent;
    Property BarStyle: TVVMStatusbarStyle read FStyle write SetFStyle default vsbColor;
    Property CenterText: Boolean read FCentered write SetCentered default False;
    { Published declarations }
  end;

Procedure Register;

implementation

Function getContrastColor(aC: TColor): TColor;
var aCAux: Integer;
    H,L,S: Integer;
    aC1: Integer;
begin
  ac1:=ColorToRGB(aC);
  acAux:=GetRValue(aC1)+GetGValue(aC1)+GetBValue(aC1);
  If (GetGValue(aC)>190) or (acAux>255*3/2) then Result:=clBlack
    else Result:=clWhite;
end;

Function GetInvertedColor(C: TColor): TColor;
var Z: Longint;
    R,G,B: Byte;
begin
  Z:=ColorToRGB(C);
  R:=GetRValue(Z);
  G:=GetGValue(Z);
  B:=GetBValue(Z);
  R:=255-R;
  G:=255-G;
  B:=255-B;
  Result:=TColorRef(RGB(R,G,B));
end;

Constructor TVVMStatusBar.Create;
begin
  Inherited Create(AOwner);
  FMin:=0;
  FMax:=100;
  FCurrent:=0;
  FNumPanel:=-1;
  FOnChangePerc:=NIL;
  FBarColor:=clBlue;
  FCaption:=Self.Name;
  FCaptStyle:=ssPercent;
  FOldRect:=Rect(0,0,0,0);
  FStyle:=vsbColor;
  FPercent:=0;
  FCentered:=False;
  FFull:=False;
end;

Function  TVVMStatusBar.GetPercent: Integer;
begin
  Try
    GetPercent:=(FCurrent-FMin)*100 div (FMax-FMin);
  except
    getPercent:=0;
  end;
end;

procedure TVVMStatusBar.SetBarColor(val: TColor);
begin
  If Val<>FBarColor then begin
    FBarColor:=Val;
    FFull:=True;
    PaintPanel(FRect);
    FFull:=False;
//    Invalidate;
  end;
end;

Procedure TVVMStatusBar.StepIt;
begin
  If FCurrent<FMax then begin
    Position:=Position+1;
  end;
end;

procedure TVVMStatusBar.SetMax(Val: Longint);
begin
  If (val>FMin) and (Val<>FMax) then begin
    FMax:=Val;
    Position:=FMin;
  end;
end;

procedure TVVMStatusBar.SetMin(Val: Longint);
begin
  If (val<FMax) and (Val<>FMin) then begin
    FMin:=Val;
    Position:=FMin;
  end;
end;

procedure TVVMStatusBar.SetPosition(Val: Longint);
begin
  If Val<FMin then Val:=FMin;
  If Val>FMax then Val:=FMax;
  If Val<>FCurrent then begin
    FCurrent:=Val;
    If FPercent<>GetPercent then begin
      FPercent:=GetPercent;
      PaintPanel(FRect);
      If Assigned(FOnChangePerc) then FOnChangePerc(Self,Percent);
    end;
  end;
end;

procedure TVVMStatusBar.SetPanel(val: Integer);
begin
  If Val<>FNumPanel then Begin
    FNumPanel:=Val;
    If Val>-1 then
      If Panels.Count>Val then
        Panels[Val].Style:=psOwnerDraw;
    Invalidate;
  end;
end;

Procedure TVVMStatusBar.SetStyle(Val: TShowStyle);
begin
  If Val<>FCaptStyle then begin
    FCaptStyle:=Val;
    Invalidate;
    FFull:=True;
    PaintPanel(FRect);
    FFull:=False;
//    PaintPanel(FRect);
  end;
end;

Procedure TVVMStatusBar.SetCaption(Val: String);
begin
  If Val<>FCaption then begin
    FCaption:=Val;
    Invalidate;
    FFull:=True;
    PaintPanel(FRect);
    FFull:=False;
  end;
end;

Procedure TVVMStatusBar.PaintPanel(const Rect: TRect);
Var DX,DX1,DX2: TRect;
    Aux: String;
    PSt: TPaintStruct;
begin
  Dx:=Rect;
  If FNumPanel=Panels.Count-1 then begin
    DX.Right:=DX.Right-GetSystemMetrics(SM_CYHSCROLL);
  end;
  Case FCaptStyle of
    ssNone    : Aux:='';
    ssPercent : Aux:=Format('%u%%',[Percent]);
    sstext    : Aux:=FCaption;
    ssBoth    : Aux:=FCaption+Format('%u%%',[Percent])
  end;
  With Canvas do begin
    PSt.hdc:=Self.Canvas.Handle;
    PSt.fErase:=False;
    PSt.rcPaint:=Rect;
    Canvas.Lock;
    If FStyle=vsbColor then Canvas.Brush.Color:=FBarColor;
    Canvas.Brush.Style:=bsSolid;
 {20.10.99}
    If FStyle=vsbColor then
//       Canvas.Font.Color:=GetInvertedColor(FBarColor)
//       else Canvas.Font.Color:=GetInvertedColor(Canvas.Brush.Color{Self.Font.Color});
       Canvas.Font.Color:=GetContrastColor(FBarColor)
       else Canvas.Font.Color:=GetContrastColor(Canvas.Brush.Color{Self.Font.Color});
 {20.10.99}
{
    Rectangle(dx.Left,dx.Top,
              dx.Left+(dx.Right-dx.Left)*percent div 100,dx.Bottom);
}
    DX1:=DX;
    DX1.Right:=dx.Left+(dx.Right-dx.Left)*percent div 100;
    if ((DX1.Left<>FOldRect.Left) or
       (DX1.Right<>FOldRect.Right) or
       (DX1.Top<>FOldRect.Top) or
       (DX1.Bottom<>FOldRect.Bottom)) or FFull then begin
       If FCentered then begin
         SetTextAlign(Canvas.Handle,TA_CENTER or VTA_CENTER);
//         InflateRect(DX1, -1, -1);
         TextRect(DX1,DX1.Left+(DX1.Right-DX1.Left) div 2,
                  DX1.Top+(DX1.Bottom-DX1.Top) div 2-TextHeight('Wg') div 2, Aux);
//         InflateRect(DX1, 1, 1);
       end
       else
         TextRect(DX1,dx.Left+(dx.Right-dx.Left) div 2-TextWidth(Aux) div 2,
                  dx.Top+(dx.Bottom-dx.Top) div 2-TextHeight(Aux) div 2,Aux);
    Canvas.Brush.Color:=clBtnFace;
 {20.10.99}
//    If FStyle=vsbColor then Canvas.Font.Color:=FBarColor;
//    If FStyle=vsbColor then Canvas.Font.Color:=GetInvertedColor(Color);
    If FStyle=vsbColor then Canvas.Font.Color:=GetContrastColor(Color);
 {20.10.99}
    DX2:=DX1;
      DX1.Left:=DX1.Right;
      DX1.Right:=Dx.right;
      If Not FCentered then
        TextRect(DX1,dx.Left+(dx.Right-dx.Left) div 2-TextWidth(Aux) div 2,
                   dx.Top+(dx.Bottom-dx.Top) div 2-TextHeight(Aux) div 2,Aux)
        else FillRect(DX1);
    If FStyle=vsbColor then FOldRect:=DX1;
    If FStyle=vsbEdged then begin
      FOldRect:=DX1;
      If FPercent>0 then
        DrawEdge(Self.Canvas.Handle, DX2, BDR_RAISEDINNER, BF_RECT);
    end;
    end;
    Canvas.Unlock;
  end;
end;

Procedure TVVMStatusBar.DrawPanel(Panel: TStatusPanel; const Rect: TRect);
begin
  Inherited DrawPanel(Panel,Rect);
  If Panel.Index<>FNumpanel then Exit;
  Panels[FNumPanel].Style:=psOwnerDraw;
  FRect:=Rect;
  PaintPanel(Rect);
{
    DrawText(Canvas.Handle,PChar(Aux),-1,Dx1,
         DT_SingleLine or DT_Center or DT_VCenter);
}
end;
Procedure TVVMStatusBar.SetFStyle(Value: TVVMStatusbarStyle);
begin
  If Value<>FStyle then begin
    FStyle:=Value;
    Invalidate;
    FFull:=True;
    PaintPanel(FRect);
    FFull:=False;
  end;
end;

Procedure TVVMStatusBar.SetCentered(Value: Boolean);
begin
  If Value<>FCentered then begin
    FCentered:=Value;
    Invalidate;
    FFull:=True;
    PaintPanel(FRect);
    FFull:=False;
  end;
end;

Procedure Register;
begin
  RegisterComponents('VVMPage',[TVVMStatusBar]);
end;

procedure TVVMStatusBar.Invalidate;
begin
  inherited;
end;

end.